/** @package Provides integration capabilities for Firefox and Surfulater.

    surfulaterOverlay.js

    License: MPL 1.1/GPL 2.0/LGPL 2.1 See License in shared.js
    Copyright(c) Soft As It Gets Pty Ltd 2004-2010

    16 Dec 2004 NF  Reworked to run our new EXE directly. See: SendToSurfulater().
    4 Nov 2005 NF   Updated to use new FF 1.5 code.
    3 Mar 2006 NF   Updated to pass <base href=...> value to SUL.
    11 Apr 2006 NF  Updated for new XML Clipboard spec.
    11 Apr 2006 NF  Added BookmarkPage() and BuildSurfulaterXferBookmark().
    29 Jul 2008 NF  Added GetClipboardText(). Updated BuildSurfulaterXfer() and
                    SendToSurfulater() to use Base64 Thumbnail image from Clipboard.
    13 Aug 2008 NF  Reworked CopyToClipboard() and SendToSurfulater() etc.
    22 Sep 2008 NF  Reworked code to use XMLHttpRequest for IPC instead of Clipboard.
    23/07/2009 NF   Updated GetSurfulaterPath() to use HKCU instead of HKLM.
    5/02/2010 NF    Major revamp to use OO and encapsulate code into our own class/namespace.

        Author: Neville Franks http://www.surfulater.com
        Created: NF  1 Dec 2004 11:01:52 AM
        Created with ED for Windows - http://www.getsoft.com

	Last change: NF 5/02/2010 1:03:41 PM
*/

// To show all trace messages uncomment: SurfulaterShared.dev = true; in <A HREF="..\Firefox\Surfulater\chrome\content\shared.js#^SurfulaterLib.prototype.trace = function">
// else set bforce param = true.
// SurfulaterShared.alertmsg() displays an alert().

// See overlay.xul which creates a Surfulater instance.

/** Implementation Note.
 *  We will never instantiate multiple instances of this class so I could use static functions
 *  and variables throughout, however I haven't doen this, instead using a class variables and
 *  a mix of class and static functions. 5/02/2010
 */
var Surfulater = function()
{
    // Class Vars
    this.gsul_path = '';
    this.gsXferText = '';
    this.gSelection = '';
    this.gRange = '';
    this.gbHadSelection = false;

    // See: <A HREF="D:\SAIG\FMNLib\FMNWebServer.h#^#define SULWebServerPort "9108"">
    var vsHTTPServerPort = "9108";
    this.urlSULWebServer = "http://localhost:" + vsHTTPServerPort + "/FirefoxContentCapture.xhr"
}

/** Returns the full path + fspec for our SurfulaterFirefox.exe
 *  @since 5/02/2010
 */
Surfulater.prototype.GetSurfulaterFSpec = function()
{
    if ( typeof( this.gsul_path ) == "undefined" || this.gsul_path.length == 0 )
    {   // should already have been handled.
        alert( "NH: GetSurfulaterFSpec() & gsul_path is undefined: " + gsul_path );
    }
    return this.gsul_path + '\\' + "SurfulaterFirefox.exe";  // "SurfulaterFirefox-nd.exe";  // Set for Debug/Release Build as req'd.
}


/** Returns the path where Surfulater is installed.
 */
Surfulater.prototype.GetSurfulaterPath = function()
{
    var sul_path = "";
    if ( SurfulaterShared.initRegistryAccess() )
    {
        // New code that looks in 'HKCU\Software\Classes\Surfulater.Document\Path' so we don't use HKLM. 23/07/2009
        // See: <A HREF=".\FMN.cpp#^CFMNApp::RegisterShellFileTypesEx">
        try
        {
            var reg_key = 2;  // 2 = HKEY_CURRENT_USER  - see <A HREF="..\Firefox\Surfulater\chrome\content\shared.js#^SurfulaterLib.prototype.getRegEntry = function">
            var reg_path = "Software\\Classes\\Surfulater.Document\\Path";
            var reg_name = "";   // '(Default)' entry
            // var reg_name = "PathAcme";  // To test exception code - or just delete the Reg key.
            sul_path = SurfulaterShared.getRegEntry( reg_key, reg_path, reg_name );
            // rem: throws an exception if reg key doesn't exist.
            SurfulaterShared.trace( "a) GetSurfulaterPath()sul_path: " + sul_path );
        }
        catch( e )
        {   // rem: Should never get here as Surfulater installs this Extension so it must have been started to do so.
            SurfulaterShared.trace( "GetSurfulaterPath() can't access Registry Key: HKCU\\Software\\Classes\\Surfulater.Document\\Path", true );
        }

        // Code above should not fail! Fall through to orig code if it did. 23/07/2009
        if ( typeof( sul_path ) == "undefined" || sul_path.length == 0 )
        {
            try
            {
                var reg_key = 3;  // 3 = HKEY_LOCAL_MACHINE
                var reg_path = "Software\\Microsoft\\Windows\\CurrentVersion\\App Paths\\Surfulater.EXE";
                var reg_name = "Path";
                sul_path = SurfulaterShared.getRegEntry( reg_key, reg_path, reg_name );
                // rem: throws an exception if reg key doesn't exist. 15 Dec 2004
                SurfulaterShared.trace( "b) GetSurfulaterPath()sul_path: " + sul_path, true );
            }
            catch( e )
            {
                SurfulaterShared.trace( "GetSurfulaterPath() can't access Registry Key: HKLM\\Software\\Microsoft\\Windows\\CurrentVersion\\App Paths\\Surfulater.EXE Path key", true );
            }
        }
    }
    else
        SurfulaterShared.alertmsg( "Unable to access the Windows Registry.\n\nPlease report this to Soft As It Gets.\n\nSee: www.surfulater.com/contact.html\n for our Contact details." );

    return sul_path;
}


/** Returns true if Surfulater is installed and has been run once.
 *  @note sets gsul_path to the folder Surfulater is installed in.
 */
Surfulater.prototype.CheckSurfulaterIsInstalled = function()
{
    SurfulaterShared.trace( "CheckSurfulaterIsInstalled()" );

    // See Implementation Notes here: <A HREF="d:\saig\Docs\FMN Design.txt#Firefox Extension Implementation">
    var bStat = true;
    this.gsul_path = this.GetSurfulaterPath();
    if ( typeof( this.gsul_path ) == "undefined" || this.gsul_path.length == 0 )
    {
        bStat = false;
        alert( "Surfulater either isn't installed on this PC or has never been started.\n\nDownload Surfulater from www.surfulater.com, install it and start it,\nand then try again.\n\nIf you see this message again close Firefox and Surfulater, then start\nSurfulater and wait for it to display, then start Firefox.\n\nContact us if the problem persists." );
        SurfulaterShared.trace( "Surfulater not found in the Registry", true );
    }
    return bStat;
}


/** Starts Surfulater.
 *  Accessed via. the Tools menu, and SendToSULComplete().
 */
Surfulater.prototype.StartSurfulater = function()
{
    if ( this.CheckSurfulaterIsInstalled() )
    {
        // var app_exe = gsul_path + "\\Surfulater-ND.EXE";
        var app_exe = this.gsul_path + "\\Surfulater.EXE";  // TODO: Set for Debug/Release Build
        var aArgs;
        // SurfulaterShared.trace( "About to run:\n" + app_exe + " " + aArgs, true );
        SurfulaterShared.RunProgram( false, app_exe, aArgs );   // no reason to block
    }
}


/** Starts Surfulater and waits for it to start.
 *  @caller SendToSurfulater().
 */
Surfulater.prototype.StartSurfulaterAndWait = function()
{
    // Bring Surfulater the Foreground as req'd
    var aArgs = ['SULStartAndWait'];
    // SurfulaterShared.trace( "In StartSurfulaterAndWait() about to run:\n" + sul_ff_exe + " " + aArgs, true );
    SurfulaterShared.RunProgram( true, this.GetSurfulaterFSpec(), aArgs );
    // rem: first param = bBlock which is true to block till .exe ends.
    // SurfulaterShared.trace( "In StartSurfulaterAndWait() returned from to running:\n" + sul_ff_exe + " " + aArgs, true );
}


/** Creates a new Surfulater Article.
 *  Accessed via. our Context menu add-in
 *  @note See Implementation Notes here: <A HREF="d:\saig\Docs\FMN Design.txt#Firefox Extension Implementation">
 */
Surfulater.prototype.NewArticle = function()
{
    this.NewArticleEx( true );
}


/** Creates a new Surfulater Article and Attaches the current Web page to it.
 *  Accessed via. our Context menu add-in
 *  @note See Implementation Notes here: <A HREF="d:\saig\Docs\FMN Design.txt#Firefox Extension Implementation">
 */
Surfulater.prototype.NewArticleAndPage = function()
{
    this.NewArticleEx( false );
}


/** Creates a new Surfulater Article and optionally Attaches the current Web page to it.
 *  @caller NewArticle() and NewArticleAndPage() only.
 */
Surfulater.prototype.NewArticleEx = function( bNewArticle )
{
    // SurfulaterShared.trace( "NewArticleEx() bNewArticle: " + bNewArticle, true );

    if ( this.CheckSurfulaterIsInstalled() )
    {
        // Grab everything we need now, to play safe. Thumbnail added r'on.
        this.gsXferText = Surfulater.BuildSurfulaterXfer( bNewArticle );

        this.CloseSelection();   // So it doesn't show up in the thumbnail

        // Wait a bit and then SendToSurfulater().
        var that = this;    // See SVN for other attempts. 5/02/2010
        setTimeout( function() { that.SendToSurfulater( true ) }, 1 );
        // See Implementation Notes here: <A HREF="d:\saig\Docs\FMN Design.txt#Firefox Extension Implementation">
    }
}


/** Attaches the current Firefox Web page to the current Surfulater Article.
 *  Accessed via. our Context menu add-in
 */
Surfulater.prototype.AttachPage = function()
{
    // SurfulaterShared.trace( "AttachPage()", true );

    if ( this.CheckSurfulaterIsInstalled() )
    {
        // Grab everything we need now, to play safe
        this.gsXferText = Surfulater.BuildSurfulaterXferForPage();
        // And sent it to SUL.
        this.SendToSurfulater( false );
    }
}


/** Creates a Bookmark in Surfulater for the current Web Page.
 *  Accessed via. our Context menu add-in
 *  @since 11 Apr 2006
 */
Surfulater.prototype.BookmarkPage = function()
{
    if ( this.CheckSurfulaterIsInstalled() )
    {
        // Grab everything we need now, to play safe
        this.gsXferText = Surfulater.BuildSurfulaterXferBookmark();
        // And sent it to SUL.
        this.SendToSurfulater( false );
    }
}


/** Close the current selection, if any.
 *  @see RestoreSelection()
 */
Surfulater.prototype.CloseSelection = function()
{
    // SurfulaterShared.trace( "In CreateSurfulaterThumbnail()", true );

    this.gbHadSelection = false;
    var focusedWindow = document.commandDispatcher.focusedWindow;
    if ( focusedWindow.getSelection )
    {
        this.gSelection = focusedWindow.getSelection();
        // SurfulaterShared.trace( "gSelection.rangeCount: " + gSelection.rangeCount, true );
        if ( this.gSelection.rangeCount > 0 )
        {
            this.gRange = this.gSelection.getRangeAt(0);   // used in RestoreSelection() to restore the current selection.
            this.gSelection.removeRange( this.gRange );
            this.gbHadSelection = true;
            // SurfulaterShared.trace( "selection removed", true );
        }
    }
}


/** Restore the Selection
 *  @see CloseSelection().
 */
Surfulater.prototype.RestoreSelection = function()
{
    if ( this.gbHadSelection )
    try
    {
        this.gSelection.addRange( this.gRange );
        // SurfulaterShared.trace( "selection.addRange() done.", true );
    }
    catch( e )
    {
        SurfulaterShared.trace( "NH!! selection.addRange() exception raised", true );
    }
}


/** Send the SXCF text to Surfulater.
 *  @see <A HREF="d:\saig\Surfulater\SurfulaterEXEC\AppExec.cpp#Firefox JS and DLL Implementation Notes">
 *  Calls CHTTPDBServer::ProcessPost().
 */
Surfulater.prototype.SendToSurfulater = function( bRestoreSelection )
{
    // SurfulaterShared.trace( "SendToSurfulater() called.", true );

    // If SUL isn't running, start it and wait for it initialize. Now works: 27 Oct 2008
    this.StartSurfulaterAndWait();
    var that = this;   // for req.onreadystatechange() below

    var req = Components.classes["@mozilla.org/xmlextras/xmlhttprequest;1"].createInstance(Components.interfaces.nsIXMLHttpRequest);
    req.open( "POST", this.urlSULWebServer, true );       // rem: Async. We can also do Sync but this is better.
    req.onreadystatechange = function ( aEvt )
    {
        // SurfulaterShared.trace( "In SendToSurfulater() req.onreadystatechange() req.readyState: " + req.readyState, true );
        if ( req.readyState == 4 )   // 4: complete
        {   // POST has completed.
            that.SendToSULComplete( req.status, bRestoreSelection );
        }
    };
    req.send( this.gsXferText );

    // SurfulaterShared.trace( "SendToSurfulater()\n" + url + "\nfinished!", true );
}


// See req.onreadystatechange() above.
Surfulater.prototype.SendToSULComplete = function( status, bRestoreSelection )
{
    SurfulaterShared.trace( "In SendToSULComplete() status: " + status, true );

    // A few users on Vista are/were getting 'status == 0' reported. I've updated the code to accept
    // 'status == 0' as valid and we'll see what happens. Google'ing indicated this is possible for
    // ftp && file protocols. Google: "XMLHttpRequest.status 0". Looks like a local Web server can for some also do this. 4 Feb 2009
    // ref: sulkb://kb=Programming,Fid=5250,Rid=6174
    if ( status == 200 || status == 0 )
    {
        // Bring Surfulater the Foreground as req'd. Must do this before SULCallSurfulater else if latter takes too long twon't com to FG.
        var args = ['SULCallSULToForeground'];
        SurfulaterShared.RunProgram( true, this.GetSurfulaterFSpec(), args );
        // rem: first param = bBlock which is true to block till .exe ends.
        // SurfulaterShared.trace( "run:\n" + sul_ff_exe_fspec + " " + args + " finished!", true );

        // Get Surfulater to create the new Article or Attachment.
        var args = ['SULCallSurfulater'];
        SurfulaterShared.RunProgram( false, this.GetSurfulaterFSpec(), args );     // don't block so clipboard is released asap.
        // SurfulaterShared.trace( "run:\n" + sul_ff_exe_fspec + " " + args + " finished!", true );
    }
    else   // removed 'else status = 0 ...' code 4 Feb 2009
    {   // new code 13 Aug 2008
        SurfulaterShared.trace( "In SendToSULComplete() and SendToSurfulater() failed!! Status: " + status, true );
        alert( "An error occurred transferring content to Surfulater.\n\nStart Surfulater if it is not already running.\n\nContact us if this problem persists.\n\nStatus: " + status );
    }

    if ( bRestoreSelection )
        this.RestoreSelection();   // rem: Only if we closed it.
}


/** Build the information we need to Attach the current Web page to the current
 *  Surfulater Article and return same.
 */
Surfulater.BuildSurfulaterXferForPage = function()
{
    // SurfulaterShared.trace( "In BuildSurfulaterXferForPage()", true );

    var focusedWindow = document.commandDispatcher.focusedWindow;
    var title = focusedWindow.document.title;
    var url = focusedWindow.location.href;

    // Get the users selection, if any, in HTML.
    // var htmlselection = getHTMLOfSelection();
    // SurfulaterShared.trace( "htmlselection: " + htmlselection, true );

    // Build 'OurPrefix + URL + Title' ready for the Clipboard.
    // Format up as XML CDATA with same field names as DataBase.
    var sText = "<SULCONTENT action=\"attach\" source=\"WebBrowser\" source_application=\"Firefox\" template=\"IE\">\n"
               + "<URL>" + Surfulater.TextToCDATA( url ) + "</URL>\n"
               + "<Reference>" + Surfulater.TextToCDATA( url ) + "</Reference>\n"
               + "<Title>" + Surfulater.TextToCDATA( title ) + "</Title>\n"
               + "<MetaDescription>" + Surfulater.TextToCDATA( Surfulater.GetMetaTagContent( focusedWindow.document, "description", true ) ) + "</MetaDescription>\n"
               + "<MetaKeywords>" + Surfulater.TextToCDATA( Surfulater.GetMetaTagContent( focusedWindow.document, "keywords", true ) ) + "</MetaKeywords>\n"
               + "<BASE>" + Surfulater.TextToCDATA( Surfulater.GetBase( focusedWindow.document ) ) + "</BASE>\n"   // 3 Mar 2006
               + "</SULCONTENT>";

    // SurfulaterShared.trace( "sText: " + sText, true );
    return sText;
}


/** Build the information we need for a new Surfulater Article and return same.
 *  Used for 'New Article' and 'New Article plus Page'.
 *
 *  @param bNewArticle true if adding a new article, else false for new article + web page.
 *
 *  @caller NewArticleEx() only.
 */
Surfulater.BuildSurfulaterXfer = function( bNewArticle )
{
    // SurfulaterShared.trace( "In BuildSurfulaterXfer()", true );

    var focusedWindow = document.commandDispatcher.focusedWindow;
    var title = focusedWindow.document.title;
    var url = focusedWindow.location.href;

    // Get the users selection, if any, in HTML.
    var htmlselection = Surfulater.getHTMLOfSelection();
    // SurfulaterShared.trace( "htmlselection: " + htmlselection, true );

    // Build 'OurPrefix + URL + Title + Selection etc' ready for the Clipboard.
    // Format up as XML CDATA with same field names as DataBase.
    var sText = "<SULCONTENT action=\"add\" source=\"WebBrowser\" source_application=\"Firefox\" template=\"IE\">\n"
               + ( bNewArticle ? "" : ( "<URL>" + Surfulater.TextToCDATA( url ) + "</URL>\n" ) )
               + "<Reference>"      // added MetaDesc to Reference 9 Feb 2004
               + Surfulater.TextToCDATA( url + "<br>" + Surfulater.GetMetaTagContent( focusedWindow.document, "description", true ) )
               + "</Reference>\n"
               + "<Title>" + Surfulater.TextToCDATA( title ) + "</Title>\n"
               + "<Text>" + Surfulater.TextToCDATA( htmlselection ) + "</Text>\n"
               + "<MetaDescription>" + Surfulater.TextToCDATA( Surfulater.GetMetaTagContent( focusedWindow.document, "description", true ) ) + "</MetaDescription>\n"
               + "<MetaKeywords>" + Surfulater.TextToCDATA( Surfulater.GetMetaTagContent( focusedWindow.document, "keywords", true ) ) + "</MetaKeywords>\n"
               + "<BASE>" + Surfulater.TextToCDATA( Surfulater.GetBase( focusedWindow.document ) ) + "</BASE>\n"
               + "</SULCONTENT>";

    // SurfulaterShared.trace( "sText: " + sText, true );
    return sText;
}


/** Build the information we need for a new Surfulater Bookmark and return same.
 *  @caller BookmarkPage().
 *  @since 11 Apr 2006
 */
Surfulater.BuildSurfulaterXferBookmark = function()
{
    // SurfulaterShared.trace( "In BuildSurfulaterXfer()", true );

    var focusedWindow = document.commandDispatcher.focusedWindow;
    var title = focusedWindow.document.title;
    var url = focusedWindow.location.href;

    // Buil 'OurPrefix + URL + Title + Text etc' ready for the Clipboard.
    // Format up as XML CDATA with same field names as DataBase.
    var sText = "<SULCONTENT action=\"add\" source=\"WebBrowser\" source_application=\"Firefox\" template=\"Bookmark\" emptytextok=\"true\" >\n"
              + "<Reference>" + Surfulater.TextToCDATA( url ) + "</Reference>\n"
              + "<Title>" + Surfulater.TextToCDATA( title ) + "</Title>\n"
              + "<Text>" + Surfulater.TextToCDATA(  Surfulater.GetMetaTagContent( focusedWindow.document, "description", true ) ) + "</Text>\n"
              + "<Keywords>" + Surfulater.TextToCDATA( Surfulater.GetMetaTagContent( focusedWindow.document, "keywords", true ) ) + "</Keywords>\n"
              + "<BASE>" + Surfulater.TextToCDATA( Surfulater.GetBase( focusedWindow.document ) ) + "</BASE>\n"   // 3 Mar 2006
              //  + "<Visited>" + "20060410T160742" + "</Visited>\n"   // for testing 'def=' code only
              + "</SULCONTENT>";
               
    // SurfulaterShared.trace( "sText: " + sText, true );
    return sText;
}


/** Returns the current selection in HTML.
 */
Surfulater.getHTMLOfSelection = function()
{
    var focusedWindow = document.commandDispatcher.focusedWindow;
    if ( focusedWindow.getSelection )
    {
        var selection = focusedWindow.getSelection();
        // SurfulaterShared.trace( "getHTMLOfSelection() selection.rangeCount: " + selection.rangeCount, true );
        if ( selection.rangeCount > 0 )
        {
            var range = selection.getRangeAt(0);
            return Surfulater.html_Text( focusedWindow.document, range );
        }
    }
    return '';
}


/** Returns the spec'd range as HTML.
 *  @see http://www.faqts.com/knowledge_base/view.phtml/aid/32427
 */
Surfulater.html_Text = function( mydocument, myrange )
{
    var div = mydocument.createElement("div")
    div.appendChild( myrange.cloneContents() );
    // SurfulaterShared.trace( "div: " + div, true );
    return div.innerHTML;
}


// Wrap text in a CDATA XML Element.
Surfulater.TextToCDATA = function( sText )
{   // rem: play safe and change any cdata end tag. Don't think we'll
    //      ever see one. See also: cdata_escape()
    sText = sText.replace( /]]>/g, "]]&gt;"  );
    sText = "<![CDATA[" + sText + "]]>";
    // SurfulaterShared.trace( "2) sText:" + sText );
    return sText;
}


// Returns the specified Meta Tag Name Content or empty string if none.
// See: <A HREF="..\Docs\FMN Design.txt#Getting Meta Tags from a Web page">
Surfulater.GetMetaTagContent = function( oDocument, sMetaName, bAll )
{
    var MetaTagList = oDocument.getElementsByTagName("meta");
    var ListLength = MetaTagList.length;
    // SurfulaterShared.trace( "GetMetaTagContent() ListLength: " + ListLength, true );

    var i, sResult = "";
    for ( i = 0; i < ListLength; i++ )
    {
        var MetaName = MetaTagList[i].name;
        // SurfulaterShared.trace( "MetaName: " + MetaName + " sMetaName: " + sMetaName );
        if ( Surfulater.isStrEqNoCase( MetaName, sMetaName ) )
        {
            // SurfulaterShared.trace( "Content:" + MetaTagList[i].content );
            if ( sResult.length > 0 )
                sResult += ", "       // append , for entries after the first.
            sResult += MetaTagList[i].content;
            if ( !bAll )
                break;
        }
    }
    return sResult;
}

// Case insensitive string compare
Surfulater.isStrEqNoCase = function( str1, str2 )
{
    return str1.toUpperCase() == str2.toUpperCase();
}


// Return the <base href=".." value if it exists.
// See: http://msdn.microsoft.com/library/default.asp?url=/workshop/author/dhtml/reference/methods/getelementsbytagname.asp
Surfulater.GetBase = function( oDocument )
{
    var oBaseColl = oDocument.getElementsByTagName("base");
    // SurfulaterShared.trace( "BASE href:" + ( oBaseColl && oBaseColl.length ) ? oBaseColl[ 0 ].href : "" );
    return ( ( oBaseColl && oBaseColl.length ) ? oBaseColl[ 0 ].href : "" );
}


/****************************************************************
*                                                               *
****************************************************************/
