/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is
 * Henrik Gemal http://gemal.dk  See: http://gemal.dk/mozilla/launchy.html
 * Heavilly Modifed by Neville Franks. www.surfulater.com
 *
 * Portions created by the Initial Developer are Copyright (C) 2001-2004
 * the Initial Developer. All Rights Reserved.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK *****

    4 Nov 2005 NF   Updated to work with FF 1.5.

	Last change: NF 4 Nov 2005 11:01:11 AM
 */


var SurfulaterLib = function() {}


/** Dump message to the JavaScript console but only if you're a developer.
 *  @param msg msg to dump.
 *  @param bforce true to force msg to be dumped.
 */
SurfulaterLib.prototype.trace = function( msg, bforce )
{
    // Set: SurfulaterShared.dev = true; to show all trace message. I would think this can be done from the FF JS Console but have failed to.
    // SurfulaterShared.dev = true;  // Uncomment as req'd

	if ( bforce || (typeof(SurfulaterShared) != "undefined" && typeof(SurfulaterShared.dev) != "undefined" && SurfulaterShared.dev ) )
    {
		var CONSOLE_SERVICE = Components.classes['@mozilla.org/consoleservice;1'].getService(Components.interfaces.nsIConsoleService);
		try {
			CONSOLE_SERVICE.logStringMessage("Surfulater: " + msg);
		} catch(ex) {
			CONSOLE_SERVICE.logStringMessage("Surfulater: Unable to log message. Type was <" + typeof( msg ) + ">");
		}
	}
}


/** Display the specified message.
 */
SurfulaterLib.prototype.alertmsg = function( msg )
{
    alert( "Surfulater error: " + msg );
}


/** Runs the specified Windows .EXE
 *
 *  @param bBlock true to block here until the .exe ends.
 *  @param executable program to run. Full path is required for Windows.
 *  @param aArgs array of arguments to pass to the executable.
 */
SurfulaterLib.prototype.RunProgram = function( bBlock, executable, aArgs )
{
    if ( executable == null )
    {
        return false; // no command is set
    }
    
    try
    {
        var exec = Components.classes["@mozilla.org/file/local;1"].
                   createInstance(Components.interfaces.nsILocalFile);
        var pr = Components.classes["@mozilla.org/process/util;1"].
                 createInstance(Components.interfaces.nsIProcess);
        var path = null;

        try
        {   // throws an exception on windows
            path = pr.getEnvironment("PATH").split(":");
            // alert( "path1: " + path );
        }
        catch (e)
        {
            // do nothing
        }
        // alert( "path2: " + path );
        
        // If executable is an absolute path, run it or fail.  If not, then
        // look for it in $PATH.
        if ( executable.charAt(0) == "/" || path == null )
        {
            exec.initWithPath( executable );
            if ( !exec.exists() )
            {
			    SurfulaterShared.trace( "executable '" + executable + "' does not exist." );
                SurfulaterShared.alertmsg( "executable '" + executable + "' does not exist." );
                return false;
            }
        } 
        else
        {   // search the paths - n/u on Windows
            var found = false;
            for ( i = 0; i < path.length; i++)
            {
                exec.initWithPath( path[i] );
                exec.appendRelativePath( executable );
                if ( exec.exists() )
                {
                    found = true;
                    break;
                }
            }

            if ( !found )
            {
                SurfulaterShared.alertmsg( "could not find '" + executable + "' in path.");
                return false;
            }
        }

        var len = 0;
        if ( aArgs )
            len = aArgs.length;
        else
            aArgs = null;

        pr.init( exec );
        pr.run( bBlock, aArgs, len );
    }
    catch (e)
    {
		SurfulaterShared.trace( "cannot run executable '" + executable + "' (aArgs: " + aArgs + "): " + e );
        SurfulaterShared.alertmsg( "cannot run executable '" + executable + "' (aArgs: " + aArgs + "): " + e );
        return false;
    }
    // alert(pr.pid); /* pid is not implemented :((( */
    return true;
}


/** Initialize access to the Windows Registry.
 */
SurfulaterLib.prototype.initRegistryAccess = function()
{
	// SurfulaterShared.trace("In SurfulaterLib.prototype.initRegistryAccess() - 1", true );   // 3 Nov 2005

	if ("@mozilla.org/windows-registry-key;1" in Components.classes)   // Added for FF V1.5
    {
		// SurfulaterShared.trace("Try using the windows-registry-key service!");
        // This is used for FF 1.5
		try
        {
			SurfulaterShared.winhooks = Components.classes["@mozilla.org/windows-registry-key;1"].getService(Components.interfaces.nsIWindowsRegKey);
			SurfulaterShared.winhooksAPI = 2;
			// SurfulaterShared.trace( "Got windows-registry-key service!" );
		}
        catch( ex )
        {
			SurfulaterShared.trace( "Unable to get windows-registry-key service!" );
		}
    }
    else
    if ("@mozilla.org/winhooks;1" in Components.classes)
    {
	    // SurfulaterShared.trace("In SurfulaterLib.prototype.initRegistryAccess() - 2", true );   // 3 Nov 2005
        // This is used for FF 1.07 and earlier
		try
        {
			SurfulaterShared.winhooks = Components.classes["@mozilla.org/winhooks;1"].getService(Components.interfaces.nsIWindowsRegistry);
			SurfulaterShared.winhooksAPI = 1;
		}
        catch( ex )
        {
			SurfulaterShared.trace( "Unable to get winhooks service!" );
		}
	}
    else
    if ("@mozilla.org/browser/shell-service;1" in Components.classes)
    {
	    // SurfulaterShared.trace("In SurfulaterLib.prototype.initRegistryAccess() - 3", true );   // 3 Nov 2005

		try
        {
			SurfulaterShared.winhooks = Components.classes["@mozilla.org/browser/shell-service;1"].getService(Components.interfaces.nsIWindowsShellService);
			SurfulaterShared.winhooksAPI = 1;
		}
        catch(ex)
        {
			SurfulaterShared.trace( "Unable to get browser shell service!" );
		}
	}
    else
    {
		SurfulaterShared.trace( "Unable to get Components. This only works on Windows!", true );
	}

	if ( typeof(SurfulaterShared.winhooks) == "undefined"
      || ( SurfulaterShared.winhooksAPI == 1 && typeof(SurfulaterShared.winhooks.getRegistryEntry) != "function" )
      || ( SurfulaterShared.winhooksAPI == 2 && typeof(SurfulaterShared.winhooks.open) != "function" )
       )
    {
		// SurfulaterShared.trace( "SurfulaterShared.winhooks: " + SurfulaterShared.winhooks, true );
		SurfulaterShared.trace( "Registry access isn't available. This only works on Windows!", true );
		SurfulaterShared.winhooks = false;
    }
	else
    {   // Success. Show what we're using
		SurfulaterShared.trace( "Using winhooksAPI version <" + SurfulaterShared.winhooksAPI + ">" );
	}
    
    return SurfulaterShared.winhooks != false;
}


/** Get the Registry value for the spec'd item.
 *  @note Throws an exception if key not found etc.
 */
SurfulaterLib.prototype.getRegEntry = function( key, path, name )
{
	// key values:
	// 0: HKEY_CLASSES_ROOT = 0x80000000
	// 1: HKEY_CURRENT_CONFIG = 0x80000005
	// 2: HKEY_CURRENT_USER = 0x80000001
	// 3: HKEY_LOCAL_MACHINE = 0x80000002
	// 4: HKEY_USERS = 0x80000003

	if ( SurfulaterShared.winhooksAPI == 1 )    // see: SurfulaterLib.prototype.initRegistryAccess() above.
    {
        // This is used for FF 1.07 and earlier
        try
        {
		    return SurfulaterShared.winhooks.getRegistryEntry( key, path, name );
            // rem: throws an exception if reg key doesn't exist. 15 Dec 2004
		}
        catch( ex )
        {
		    SurfulaterShared.trace( "SurfulaterLib.getRegEntry() SurfulaterLib.winhooks.getRegistryEntry( " + key + ", " + path + ", " + name + " ) exception raised.", true );
            throw ex;
		}
	}
    else
    if ( SurfulaterShared.winhooksAPI == 2 )
    {
        // This is used for FF 1.5
		var regkey = Components.classes["@mozilla.org/windows-registry-key;1"].createInstance(Components.interfaces.nsIWindowsRegKey);
		try
        {
	        var reg = new Array(0x80000000, 0x80000005, 0x80000001, 0x80000002, 0x80000003);
			regkey.open( reg[ key ], path, Components.interfaces.nsIWindowsRegKey.ACCESS_READ );
            // rem: throws an exception if reg key doesn't exist. 15 Dec 2004
			if ( regkey.valueCount )
            {
				try
                {
					return regkey.readStringValue( name );
				}
                catch( ex )
                {
		            SurfulaterShared.trace( "SurfulaterLib.getRegEntry() regkey.readStringValue( " + name + " ) exception raised.", true );
                    throw ex;
				}
			}
		}
        catch( ex )
        {
		    SurfulaterShared.trace( "SurfulaterLib.getRegEntry() regkey.open( " + reg[ key ] + " ) exception raised.", true );
            throw ex;
		}
	}
    else
    {
		SurfulaterShared.trace( "NH: SurfulaterLib.getRegEntry() Unknown winhooksAPI value.", true );
	}
	return false;
}

// create the object
const SurfulaterShared = new SurfulaterLib();


/****************************************************************
*                                                               *
****************************************************************/
